<?php
/*
 * Main payPoint Payment class
 *
 * Author: VanboDevelops
 * Author URI: http://www.vanbodevelops.com
 *
 *	Copyright: (c) 2012 - 2014 VanboDevelops
 *	License: GNU General Public License v3.0
 *	License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */

if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! class_exists( 'WC_Payment_Gateway' ) ) { return; }

class WC_Gateway_PayPoint extends WC_Payment_Gateway {

	public function __construct() {

		$this->id			= 'paypoint';
		$this->icon			= apply_filters('woocommerce_paypoint_icon', WC_PayPoint::plugin_url() . '/paypoint_logo.png');
		$this->has_fields		= false;
		$this->liveurl			= 'https://secure.metacharge.com/mcpe/purser';
		$this->liveurl_h		= 'https://www.secpay.com/java-bin/ValCard';
		$this->method_title		= __( 'PayPoint', WC_PayPoint::TEXT_DOMAIN );

		// Load the form fields.
		$this->init_form_fields();

		// Load the settings.
		$this->init_settings();

		// Define user set variables
		$this->integration		= isset( $this->settings['integration'] )		? $this->settings['integration']	: 'hosted_ima';
		$this->title			= isset( $this->settings['title'] )		? $this->settings['title']		: 'PayPoint';
		$this->description 		= isset( $this->settings['description'] )		? $this->settings['description']	: 'PayPoint descriptions here.';
		$this->testmode			= isset( $this->settings['testmode'] )		? $this->settings['testmode']		: 'test_success';
		$this->enabled			= isset( $this->settings['enabled'] )		? $this->settings['enabled']		: 'yes';
		$this->intInstID		= isset( $this->settings['intInstID'] )		? $this->settings['intInstID']		: '';
		$this->return_URL		= isset( $this->settings['return_URL'] )		? $this->settings['return_URL']		: 'yes';
		$this->merchant_h		= isset( $this->settings['merchant_h'] )		? $this->settings['merchant_h']		: '';
		$this->remotepassword_h		= isset( $this->settings['remotepassword_h'] )	? $this->settings['remotepassword_h']	: '';
		$this->digest_key_h		= isset( $this->settings['digest_key_h'] )	? $this->settings['digest_key_h']	: '';
		$this->pp_logo_h		= isset( $this->settings['pp_logo_h'] )		? $this->settings['pp_logo_h']		: '';
		$this->mail_customer		= isset( $this->settings['mail_customer'] )	? $this->settings['mail_customer']	: '';
		$this->receive_error_logs	= isset( $this->settings['receive_error_logs'] ) ? $this->settings['receive_error_logs'] : 'yes';
		$this->emailto_error_logs	= isset( $this->settings['emailto_error_logs'] ) ? $this->settings['emailto_error_logs'] : '';
		$this->debug			= isset( $this->settings['debug'] )		? $this->settings['debug']		: 'no';
		$this->is_wc_2			= ( defined('WOOCOMMERCE_VERSION') && version_compare(WOOCOMMERCE_VERSION, '2.0', '>=') );
		$this->enable_3d_secure		= isset( $this->settings['enable_3d_secure'] )	? $this->settings['enable_3d_secure']	: 'no';
		$this->mpi_merchant_name	= isset( $this->settings['mpi_merchant_name'] )	? $this->settings['mpi_merchant_name']	: '';
		$this->mpi_merchant_url		= isset( $this->settings['mpi_merchant_url'] )	? $this->settings['mpi_merchant_url']	: '';

		// Process IPNs
		add_action( 'woocommerce_api_' . strtolower( get_class( $this ) ), array( $this, 'check_paypoint_response' ) );
		add_action( 'woocommerce_api_' . strtolower( get_class( $this ) ), array( $this, 'check_paypoint_response_h' ) );
		add_action( 'valid-paypoint-request-origins', array( $this, 'successful_request') );
		add_action( 'valid-paypoint-request-origins_h', array( $this, 'successful_request_h') );
		add_action( 'woocommerce_receipt_'. $this->id, array( $this, 'receipt_page') );

		// Save options
		add_action( 'woocommerce_update_options_payment_gateways', array( $this, 'process_admin_options') );
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

	}

	/**
	* Admin Panel Options
	*/
	public function admin_options() {

		if ( 'hosted' == $this->integration ) { ?>

			<h3><?php _e( 'PayPoint <strong>Hosted</strong>', WC_PayPoint::TEXT_DOMAIN ); ?></h3>

	<?php	} elseif ( 'hosted_ima' == $this->integration ) { ?>

			<h3><?php _e( 'PayPoint <strong>Hosted + IMA</strong>', WC_PayPoint::TEXT_DOMAIN ); ?></h3>

	<?php	} ?>

	    <p><?php _e( 'Activate PayPoint integration on your web store. It will send the user to the PayPoint website where he/she will make the payment '
		    . 'and be redirected back to your site for confirmation of your order.', WC_PayPoint::TEXT_DOMAIN ); ?></p>

	    <table class="form-table">
	    <?php
		    // Generate the HTML For the settings form.
		    $this->generate_settings_html();

	    ?>
	    </table><!--/.form-table-->
	    <?php
	} // End admin_options()

	/**
	 * Initialise Gateway Settings Form Fields
	 */
	function init_form_fields() {
		$options= array();
		$options = get_option('woocommerce_'. $this->id .'_settings');

		if ( ! empty($options['integration']) ) {
			if ( $options['integration'] == '0' ) {
				$integration = 'hosted';
			} elseif ( $options['integration'] == '1' ) {
				$integration = 'hosted_ima';
			} else {
				$integration = $options['integration'];
			}
		} else {
			$integration = 'hosted_ima';
		}

		// General settings
		$this->form_fields = array(
			'integration' => array(
				'title'		=> __( 'Integration Method', WC_PayPoint::TEXT_DOMAIN ),
				'type'		=> 'select',
				'description'	=> __( 'Choose your integration method depending on what type of account you have.', WC_PayPoint::TEXT_DOMAIN ),
				'options'	=> array('hosted' => 'Hosted', 'hosted_ima' => 'Hosted+IMA' ),
				'class'		=> 'chosen_select',
				'css'		=> 'min-width:350px;',
				'default'	=> 'hosted_ima',
			),
			'enabled' => array(
				'title'		=> __( 'Enable/Disable', WC_PayPoint::TEXT_DOMAIN ),
				'type'		=> 'checkbox',
				'label'		=> __( 'Enable PayPoint', WC_PayPoint::TEXT_DOMAIN ),
				'default'	=> 'yes'
			),
			'testmode' => array(
				'title'		=> __( 'Test Mode', WC_PayPoint::TEXT_DOMAIN ),
				'type'		=> 'select',
				'description'	=> '',
				'options'	=> array('live' => 'Live', 'test_success' => 'Testmode-Success', 'test_fail' => 'Testmode-Fail' ),
				'class'		=> 'chosen_select',
				'css'		=> 'min-width:350px;',
				'default'	=> 'test_success'
			),
			'title' => array(
				'title'		=> __( 'Method Title', WC_PayPoint::TEXT_DOMAIN ),
				'type'		=> 'text',
				'description'	=> __( 'This controls the title which the user sees during checkout.', WC_PayPoint::TEXT_DOMAIN ),
				'default'	=> __( 'PayPoint', WC_PayPoint::TEXT_DOMAIN )
			),
			'description' => array(
				'title'		=> __( 'Description', WC_PayPoint::TEXT_DOMAIN ),
				'type'		=> 'textarea',
				'description'	=> __( 'This controls the description which the user sees during checkout.', WC_PayPoint::TEXT_DOMAIN ),
				'default'	=> __("PayPoint descriptions here.", WC_PayPoint::TEXT_DOMAIN)
			),
		);

		if ( 'hosted' == $integration ) {

			$this->form_fields += array(
				'merchant_h' => array(
					'title'		=> __( 'Merchant ID', WC_PayPoint::TEXT_DOMAIN ),
					'type'		=> 'text',
					'description'	=> __( 'Enter here your merchant ID this is PayPoint.net username (usually six letters followed by two numbers).', WC_PayPoint::TEXT_DOMAIN ),
					'default'	=> ''
				),
				'remotepassword_h' => array(
					'title'		=> __( 'Remote Password', WC_PayPoint::TEXT_DOMAIN ),
					'type'		=> 'password',
					'description'	=> __( "Enter here the remote password used for md5 authantication from you to PayPoint. The remote password can be configured from within the PayPoint.net Merchant Extranet (Click on 'Change Remote Passwords' and select 'Remote' from the drop down list).", WC_PayPoint::TEXT_DOMAIN ),
					'default'	=> ''
				),
				'digest_key_h' => array(
					'title'		=> __( 'Digest Key', WC_PayPoint::TEXT_DOMAIN ),
					'type'		=> 'password',
					'description'	=> __( "Enter here the digest key used for md5 authantication from PayPoint to you. The digest key can be configured from within the PayPoint.net Merchant Extranet (Click on 'Change Remote Passwords' and select 'Digest Key' from the drop down list).", WC_PayPoint::TEXT_DOMAIN ),
					'default'	=> ''
				),
				'pp_logo_h' => array(
					'title'		=> __( 'Payment Page Logo', WC_PayPoint::TEXT_DOMAIN ),
					'type'		=> 'text',
					'description'	=> __( "Your logo must be 750 pixels in width by 100 pixels in height and must be sent to support@paypoint.net with your account ID and a message to tell them to add this logo to your payment page. Once they have uploaded the image to their secure servers and let you know it is available, you can paste the absolute URL link to it. Example: https://www.secpay.com/users/abcdef01/logo.jpg", WC_PayPoint::TEXT_DOMAIN ),
					'default'	=> ''
				),
				'mail_customer'	=> array(
					'title'		=> __( 'Send Confirmation Email to Cardholder', WC_PayPoint::TEXT_DOMAIN ),
					'type'		=> 'checkbox',
					'description'	=> __( 'Enable if you want PayPoint to send an order confirmation email to the cardholder.', WC_PayPoint::TEXT_DOMAIN ),
					'default'	=> 'yes',
				),
				'risk_title' => array(
					'title'		=> __( '3D Secure Settings', WC_PayPoint::TEXT_DOMAIN ),
					'type'		=> 'title',
					'description'	=> __( 'Settings to enable and setup PayPoint 3D secure feature. You need to enable this feature with PayPoint.net as well.', WC_PayPoint::TEXT_DOMAIN ),
				),
				'enable_3d_secure' => array(
					'title'		=> __( 'Enable 3D Secure', WC_PayPoint::TEXT_DOMAIN ),
					'type'		=> 'checkbox',
					'default'	=> 'yes',
					'description'	=> __( 'This option will send the required information to PayPoint.net in order to take advantage of its 3D Secure feature.', WC_PayPoint::TEXT_DOMAIN ),
				),
				'mpi_merchant_name' => array(
					'title'		=> __( 'Merchant Name', WC_PayPoint::TEXT_DOMAIN ),
					'type'		=> 'text',
					'default'	=> '',
					'description'	=> __( 'Enter here the merchant name to be displayed on the authentication page. Up to 25 characters.', WC_PayPoint::TEXT_DOMAIN ),
				),
				'mpi_merchant_url' => array(
					'title'		=> __( 'Merchant URL', WC_PayPoint::TEXT_DOMAIN ),
					'type'		=> 'text',
					'default'	=> '',
					'description'	=> __( 'Enter here a fully qualified URL of your merchant website. Please enter the URL with the http:// in front. ( Example: http://www.example.com )', WC_PayPoint::TEXT_DOMAIN ),
				),
			);
		} elseif ( 'hosted_ima' == $integration ) {
			$this->form_fields += array(
				'intInstID' => array(
					'title'		=> __( 'Installation ID', WC_PayPoint::TEXT_DOMAIN ),
					'type'		=> 'text',
					'description'	=> __( 'Enter here your preferred installation number, which is obtained via the Merchant Extranet: Account Management > Installations.', WC_PayPoint::TEXT_DOMAIN ),
					'default'	=> ''
				),
				'return_URL' => array(
					'title'		=> __( 'Return/Response URL', WC_PayPoint::TEXT_DOMAIN ),
					'label'		=> __( 'URL to copy: <strong>'. add_query_arg( 'wc-api', get_class( $this ), WC_Compat_PayPoint::force_https( home_url( '/' ) ) ) .'</strong>', WC_PayPoint::TEXT_DOMAIN ),
					'css'		=> 'display:none;',
					'type'		=> 'checkbox',
					'description'	=> __( "The PRN is enabled and configured on a per-installation basis via the Merchant Extranet. Go to <strong>Account Management -> Installations -> select the relevant (Hosted +IMA) installation -> Copy and Paste the URL to 'Response URL' and to the 'Return URL' fields</strong>.", WC_PayPoint::TEXT_DOMAIN ),
					'default'	=> 'yes'
				),
			);

		}

		// Debug settings
		$this->form_fields += array(
			'debug_title' => array(
				'title'		=> __( 'Debug Settings', WC_PayPoint::TEXT_DOMAIN ),
				'type'		=> 'title',
				'description'	=> '',
			),
			'receive_error_logs' => array(
				'title'		=> __( 'Receive Error Logs', WC_PayPoint::TEXT_DOMAIN ),
				'type'		=> 'checkbox',
				'default'	=> 'yes',
				'description'	=> __( 'Do you want to receive the error logs from WooCommerce security/fraud checks. You can leave this option on in Live mode and it will report any error payments to you.' ),
			),
			'emailto_error_logs' => array(
				'title'		=> __( 'Email errors logs to', WC_PayPoint::TEXT_DOMAIN ),
				'type'		=> 'text',
				'description'	=> __( "Email you want to receive all error logs to.<br /> If email field is empty, the admin email will be used.", WC_PayPoint::TEXT_DOMAIN ),
				'default'	=> get_bloginfo('admin_email'),
			),
			'debug' => array(
				'title'		=> __( 'Debug Log', WC_PayPoint::TEXT_DOMAIN ),
				'type'		=> 'checkbox',
				'label'		=> __( 'Enable Debug Log. Recommended: Test Mode only', WC_PayPoint::TEXT_DOMAIN ),
				'default'	=> 'no',
				'description'	=> __( 'Debug log will provide you with most of the data and events generated by the payment process. Logged inside <code>woocommerce/logs/PayPoint-'. sanitize_file_name( wp_hash( 'PayPoint' ) ) .'.txt</code>.', WC_PayPoint::TEXT_DOMAIN ),
			),
		);

	} // End init_form_fields()

	/**
	 * Prepare PayPoint arguments
	 *
	 * @param object $order The object of the current order
	 * @return array The array of all parameters needed for a call to PayPoint
	 **/
	function get_paypoint_params( WC_Order $order ) {

		$order_id = WC_PayPoint::get_order_number( $order );

		//Debug log
		if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Generating payment form for order #' . $order_id );

		/**
		 * Set up Hosted form parameters
		 */
		if ( 'hosted' == $this->integration ) {

			$paypoint_args = $this->get_hosted_parameters( $order );

		/**
		 * SET UP HOSTED+IMA PAYMENT FORM
		 */
		} elseif ( 'hosted_ima' == $this->integration ) {

			$paypoint_args = $this->get_hosted_ima_parameters( $order );

		} //End set up Hosted+IMA form parameters

		return apply_filters( 'woocommerce_paypoint_args', $paypoint_args, $order->id );
	}

	/**
	 * Generate the PayPoint form
	 **/
	function generate_paypoint_form( $order_id ) {

		$order = new WC_Order( $order_id );

		// Get the correct URL
		if ( 'hosted' == $this->integration ) {
			$paypoint_adr = $this->liveurl_h;
		} elseif ( 'hosted_ima' == $this->integration ) {
			$paypoint_adr = $this->liveurl;
		}

		// Get the request parameters
		$paypoint_args = $this->get_paypoint_params( $order );

		$paypoint_form_array = array();
		$option_str = '';

		//Debug log
		if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Submitted parameters: '. print_r( $paypoint_args, true ) );

		foreach ( $paypoint_args as $key => $value ) {
			if ( 'merchant_logo' == $key ) {
				$option_str .= $key .'='. $value .',';
				continue;
			}
			if ( 'cb_flds' == $key ){
				$option_str .= $key .'='. $value .',';
				continue;
			}

			$paypoint_form_array[] = '<input type="hidden" name="'.esc_attr( $key ).'" value="'.esc_attr( $value ).'" />';
		}

		if ( ! empty($option_str) ) {
			$option_str = substr( $option_str, 0, -1 );
			$paypoint_form_array[] = '<input type="hidden" name="options" value="'.$option_str.'" />';
		}

		WC_Compat_PayPoint::wc_include_js('
			jQuery("body").block({
				message: "<img src=\"' . esc_url( WC_Compat_PayPoint::force_https( WC_Compat_PayPoint::get_wc_global()->plugin_url() ) ) . '/assets/images/ajax-loader.gif\" alt=\"Redirecting...\" style=\"float:left; margin-right: 10px;\" />' . __( 'Thank you for your order. We are now redirecting you to PayPoint to make payment.', WC_PayPoint::TEXT_DOMAIN ) . '",
				overlayCSS:
				{
				    background: "#fff",
				    opacity: 0.6
				},
				css: {
				    padding:        20,
				    textAlign:      "center",
				    color:          "#555",
				    border:         "3px solid #aaa",
				    backgroundColor:"#fff",
				    cursor:         "wait",
				    lineHeight:		"32px"
				}
			    });
			jQuery("#submit_paypoint_payment_form").click();
		');

		return '<form action="'.esc_url( $paypoint_adr ).'" method="post" id="paypoint_payment_form" target="_top">
			' . implode( '', $paypoint_form_array ) . '
			<input type="submit" class="button-alt" id="submit_paypoint_payment_form" value="' . __( 'Pay via PayPoint', WC_PayPoint::TEXT_DOMAIN ) . '" /> <a class="button cancel" href="' . esc_url( $order->get_cancel_order_url() ) . '">'.__('Cancel order &amp; restore cart', WC_PayPoint::TEXT_DOMAIN ) . '</a>
		    </form>';
	}

	private function get_hosted_parameters( WC_Order $order ) {

		$order_number = WC_PayPoint::get_order_number( $order );

		//Debug log
		if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Integration method: Hosted. Generating parameter...');

		$amount = number_format( $order->get_total(), 2, '.', '' );
		$currency = get_woocommerce_currency();

		$paypoint_args = array(
			'merchant'		=> $this->merchant_h,
			'trans_id'		=> $order_number,
			'amount'		=> $amount,
			'callback'		=> add_query_arg( 'payPointListener', 'payPointResponse', add_query_arg( 'wc-api', 'WC_Gateway_PayPoint', WC_Compat_PayPoint::force_https( $this->get_return_url( $order ) ) ) ),
			'currency'		=> $currency,
			'order_key'		=> $order->order_key,
			'order_id'		=> $order->id,

			'bill_name'		=> $order->billing_first_name .' '. $order->billing_last_name,
			'bill_addr_1'		=> $order->billing_address_1,
			'bill_addr_2'		=> $order->billing_address_2,
			'bill_city'		=> $order->billing_city,
			'bill_state'		=> $order->billing_state,
			'bill_post_code'	=> $order->billing_postcode,
			'bill_country'		=> $order->billing_country,
			'bill_tel'		=> $order->billing_phone,
		);

		$desc = '';
		if ( 0 < sizeof( $order->get_items() ) ) {
			foreach ( $order->get_items() as $item ) {

				if ( $item['qty'] ) {

					$product = $order->get_product_from_item( $item );

					$item_name  = $item['name'];

					$item_meta = WC_Compat_PayPoint::get_order_item_meta( $item );

					if ( $meta = $item_meta->display( true, true ) ) {
						$item_name .= ' ('.$meta.')';
					}

					$desc .= $item['qty'] .' x '. $item_name .', ';

					if ( WC_Compat_PayPoint::is_wc_2_1() ) {
						$item_price = $order->get_item_total( $item );
					} else {
						$item_price = ( $order->prices_include_tax ) ? $order->get_item_total( $item, true ) : $order->get_item_total( $item );
					}
					$prod_name = str_replace( '_' , '', $item_name);
					// Add order product details
					$order_desc .= 'prod=' . $prod_name . ',';
					$order_desc .= 'item_amount='. $item_price;
					$order_desc .= 'x' . $item['qty'] .';';
				}
			}
			//

			// Remove the last delimiters
			$desc = substr( $desc, 0, -2 );

			// Limit the description to 125 characters
			$desc = substr( $desc, 0, 125 );

			// Remove the last delimiters
			$order_desc = substr( $order_desc, 0, -1 );
		}

		// Add the order details parameter
		$paypoint_args['order'] = $order_desc;

		if ( 'yes' == $this->enable_3d_secure ) {

			// Add the brief order description.
			$paypoint_args['mpi_description'] = $desc;

			// Add merchant name
			if ( empty( $this->mpi_merchant_name ) ) {
				$paypoint_args['mpi_merchant_name'] = substr( get_bloginfo( 'name' ), 0, 25 );
			} else {
				$paypoint_args['mpi_merchant_name'] = substr( $this->mpi_merchant_name, 0, 25 );
			}

			// Add the merchant url.
			if ( empty( $this->mpi_merchant_url ) ) {
				$paypoint_args['mpi_merchant_url'] = home_url();
			} else {
				if ( 0 !== strpos( $this->mpi_merchant_url, 'http://' ) && 0 !== strpos( $this->mpi_merchant_url, 'https://' ) ) {
					$paypoint_args['mpi_merchant_url'] = 'http://'. $this->mpi_merchant_url ;
				}
			}

			if ( 'test_success' == $this->testmode || 'test_fail' == $this->testmode ) {
				$paypoint_args['test_mpi_status'] = 'true';
			}

		}

		if ( 'yes' == $this->mail_customer ) {
			$paypoint_args['mail_customer'] = 'true';
		} else {
			$paypoint_args['mail_customer'] = 'false';
		}

		if ( ! WC_Compat_PayPoint::is_wc_2() ) {
			$paypoint_args['backcallback'] = $order->get_cancel_order_url();
			$paypoint_args['show_back'] = __( 'Cancel', WC_PayPoint::TEXT_DOMAIN );
		}

		if ( ! empty( $this->pp_logo_h ) ) {
			$paypoint_args['merchant_logo'] = "<img src='$this->pp_logo_h' height='100px' width='750px' class='floatright'>";
		}

		if ( is_ssl() ) {
			$paypoint_args['ssl_cb'] = 'true';
		}

		$paypoint_args['cb_flds'] = 'order_key:order_id';

		if ( 'test_success' == $this->testmode ) {
			$paypoint_args['test_status'] = 'true';
		} elseif ( 'test_fail' == $this->testmode ) {
			$paypoint_args['test_status'] = 'false';
		} elseif ( 'live' == $this->testmode ) {
			$paypoint_args['test_status'] = 'live';
		}

		if ( 'live' != $this->testmode ) {
			$paypoint_args['dups'] = 'false';
		}

		$hash = md5( $paypoint_args['trans_id'] . $amount . $this->remotepassword_h );

		$paypoint_args['digest'] = $hash;

		return $paypoint_args;
	}

	private function get_hosted_ima_parameters( WC_Order $order ) {

		$order_number = WC_PayPoint::get_order_number( $order );

		//Debug log
		if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Integration method: Hosted + IMA. Generating parameter...' );

		$amount = number_format( $order->get_total(), 2, '.', '' );
		$currency = get_woocommerce_currency();

		$paypoint_args = array(
			'intInstID'     => $this->intInstID,
			'strCartID'     => $order->id . ':' . $order->order_key . ':' . $order_number,
			'fltAmount'     => $amount,
			'strCurrency'   => $currency,
			'strCardHolder' => $order->billing_first_name .' '. $order->billing_last_name,
			'strAddress'    => $order->billing_address_1 .' '. $order->billing_address_2,
			'strCity'       => $order->billing_city,
			'strState'      => $order->billing_state,
			'strPostcode'   => $order->billing_postcode,
			'strCountry'    => $order->billing_country,
			'strTel'        => $order->billing_phone,
			'PT_invoice'    => $order->order_key,
			'PT_payPointListener' => 'payPointResponse',
		);

		if ( 'test_success' == $this->testmode ) {
			$paypoint_args['intTestMode'] = 1;
		} elseif ( 'test_fail' == $this->testmode ) {
			$paypoint_args['intTestMode'] = 2;
		} elseif ( 'live' == $this->testmode ) {
			$paypoint_args['intTestMode'] = 0;
		}

		// Cart Contents - Generate cart description
		$item_loop = 0;
		$desc = '';
		if ( 0 < sizeof( $order->get_items() ) ) {
			foreach ( $order->get_items() as $item ) {
				if ( $item['qty'] ) {

					$item_loop++;
					$item_name 	= $item['name'];

					$item_meta = WC_Compat_PayPoint::get_order_item_meta( $item );

					if ( $meta = $item_meta->display( true, true ) ) {
						$item_name .= ' ('.$meta.')';
					}
					$desc .= sprintf( __('Payment for Order# %s: ',WC_PayPoint::TEXT_DOMAIN), $order_number );
					$desc .= $item['qty'] .' x '. $item_name . ', ';

				}
			}
			//Add the description
			$desc = substr($desc, 0, -2 );
			$paypoint_args['strDesc'] = htmlspecialchars( $desc );
		}

		//Hash the main parameters for extra security
		$hash = $this->hash_paypoint_params( $paypoint_args );

		$paypoint_args['PT_hash'] = $hash;

		return $paypoint_args;
	}

	/**
	 * Process the payment
	 **/
	function process_payment( $order_id ) {
		$order = new WC_Order( $order_id );
		return array(
			'result' 	=> 'success',
			'redirect'	=> $order->get_checkout_payment_url( true )
		);
	}

	/**
	 * Reciept page
	 **/
	function receipt_page( $order ) {
		echo '<p>'.__('Thank you for your order, please click the button below to pay with PayPoint.', WC_PayPoint::TEXT_DOMAIN).'</p>';

		echo $this->generate_paypoint_form( $order );
	}

    /*=================================================
    * HANDLE THE PAYMENT RESPONSE HOSTED ACCOUNT
    *=================================================*/

	/**
	 * Catch the PayPoint Response
	 **/
	function check_paypoint_response_h() {

		if ( WC_PayPoint::get_get( 'payPointListener' ) == 'payPointResponse' ) {

			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Hosted: Received Response GET: '. print_r( $_GET, true ) );

			do_action( "valid-paypoint-request-origins_h", $_GET );
		}

	}

	/**
	 * Successful Payment!
	 **/
	function successful_request_h( $posted ) {

		$posted = stripslashes_deep( $posted );

		$invoice        = WC_PayPoint::get_get('order_key');
		$order_id        = WC_PayPoint::get_get('order_id');
		$currency       = WC_PayPoint::get_get('currency');
		$shopCurrency   = get_woocommerce_currency();
		$amount         = WC_PayPoint::get_get('amount');
		$tranStatus     = WC_PayPoint::get_get('code');
		$test_status    = WC_PayPoint::get_get('test_status');
		$failMessage    = WC_PayPoint::get_get('message');
		$receivedHash   = WC_PayPoint::get_get('hash');

		$error = array();

		$order = new WC_Order( (int) $order_id );

		$hash = $this->hash_paypoint_params_h();

		if ( $hash != $receivedHash ) {
			$error['invalid_hash'] = sprintf( __( 'The hash security check did not pass. The hashes generated are different.'
				. ' Hash received: %s, generated hash: %s.', WC_PayPoint::TEXT_DOMAIN ), $receivedHash, $hash );

			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: The hashes generated are different. Hash received: ' . $receivedHash . ', generated hash: ' . $hash );
		}

		$redirect_url = $this->get_return_url( $order );

		// Check amounts
		if ( 'A' == $tranStatus ) {
			if ( $amount != $order->get_total() ) {
				$error['invalid_amount'] = sprintf( __( 'The amount paid (%s) does not match the order total (%s).', WC_PayPoint::TEXT_DOMAIN ), $amount, $order->get_total() );
				//Debug log
				if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: Invalid amount.  The amount paid: ' . $amount . ' does not match the order total: ' . $order->get_total() );
			}
		}

		// Check currency.
		if ( ! empty( $currency ) ) {
			if ( $currency != $shopCurrency ) {
				$error['invalid_currency'] = sprintf( __( 'The currency paid in (%s) did not match your store currency (%s).', WC_PayPoint::TEXT_DOMAIN ), $currency, $shopCurrency );
				//Debug log
				if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: Invalid Currency.  The currency paid in: ' . $currency . ' does not match your store currency: ' . $shopCurrency );
			}
		}

		// Check invoice
		if ( $invoice != $order->order_key ) {
			$error['invalid_invoiceNumber'] = sprintf( __( 'The order invoice number did not match. Invoice received: %s, Order invoice: %s', WC_PayPoint::TEXT_DOMAIN ), $invoice, $order->order_key );

			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: Invalid Invoide Number.  Invoice received: ' . $invoice . ' Order invoice: ' . $order->order_key );
		}

		// Check if the transaction is the same test status as your store
		if ( ( 'false' == $test_status || 'true' == $test_status ) && 'live' == $this->testmode ) {
			$error['invalid_Testmode'] = __( 'You cannot make test payments, when PayPoint is set to live mode.', WC_PayPoint::TEXT_DOMAIN );

			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: Test transaction in Live mode.  You cannot make test payments when PayPoint is set to live mode.' );
		}

		// Proccess the order
		if ( empty( $error ) ) {
			if ( 'A' == $tranStatus ) {
				$order->add_order_note( __( 'PayPoint payment completed.', WC_PayPoint::TEXT_DOMAIN ) );

				$order->payment_complete();

				// Remove cart
				WC_Compat_PayPoint::empty_cart();

				//Debug log
				if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Hosted Payment Completed.' );
			} else {
				$order->add_order_note( __( 'PayPoint payment failed. Bank fail message: '. $failMessage, WC_PayPoint::TEXT_DOMAIN ) );

				//Change status Failed
				$order->update_status( 'failed' );

				//Debug log
				if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Hosted Payment Failed. Bank fail message: '. $failMessage );
			}
		} elseif ( ! empty($error) && $this->receive_error_logs == 'yes' ) {
			//Send error logs or checks if enabled
			$info = sprintf( __( 'Order #: %s ', WC_PayPoint::TEXT_DOMAIN ), $order->id );
			$this->email_paypoint_error_logs( $error, $posted, $info );
		}

		//Debug log
		if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Redirecting to: '. $redirect_url );
		echo '<meta http-equiv="refresh" content="0;url='. $redirect_url .'" />';
		exit;
	}

	/**
	 * HOSTED PAYMENT
	 * Hash the main response parameters
	 */
	function hash_paypoint_params_h(){
		$hash_string = ! empty( $_SERVER['REQUEST_URI'] ) ? $_SERVER['REQUEST_URI'] : '';

		if ( ! empty( $hash_string ) ) {
			// Remove the hash from the back of the string
			$hash_string = substr( $hash_string, 0, -37 );
		}

		// Add the Secret at the back
		$hash_string .= $this->digest_key_h;

		if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'The Hash String: '. $hash_string );

		$hash = md5( $hash_string );

		return $hash;
	}

   /*=================================================
    * HANDLE THE PAYMENT RESPONSE HOSTED+IMA ACCOUNT
    *=================================================*/

	/**
	 * Catch the PayPoint Response
	 **/
	function check_paypoint_response() {
		if ( WC_PayPoint::get_post( 'PT_payPointListener' ) == 'payPointResponse' ) {

			$_POST = stripslashes_deep( $_POST );

			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Hosted + IMA Response received:  '. print_r( $_POST, true ) );

			do_action( "valid-paypoint-request-origins", $_POST );

		}

		// If returning customer
		if ( WC_PayPoint::get_post( 'strCartID' ) && null == WC_PayPoint::get_post( 'PT_payPointListener' ) ) {

			$_POST = stripslashes_deep( $_POST );

			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Hosted + IMA Response received:  '. print_r( $_POST, true ) );

			$args = explode(':', WC_PayPoint::get_post( 'strCartID' ) );

			if ( 0 == WC_PayPoint::get_post( 'intStatus' ) ) {
				$redirect = get_permalink( WC_Compat_PayPoint::wc_get_page_id( 'cart' ) );
			} else {
				$order = new WC_Order( (int) $args[0] );

				$redirect = $this->get_return_url( $order );
			}

			wp_redirect( $redirect );
			exit;
		}
	}

	/**
	 * Successful Payment!
	 **/
	function successful_request( $posted ) {
		$hash	= $this->hash_paypoint_params( $posted );
		$intInstID  = $posted['intInstID'];
		$strCartID  = $posted['strCartID'];
		$invoice    = $posted['PT_invoice'];
		$currency   = $posted['strCurrency'];
		$shopCurrency = get_woocommerce_currency();
		$amount     = $posted['fltAmount'];
		$tranStatus = $posted['intStatus'];
		$receivedHash = $posted['PT_hash'];
		$testmode   = $posted['intTestMode'];
		$transID    = $posted['intTransID'];
		$failMessage = $posted['strMessage'];

		// Extract order number
		$split = explode( ':', $strCartID );
		$order_id = $split[0];

		$error = array();

		$order = new WC_Order( (int) $order_id );

		// Check the generated hash
		if ( $hash != $receivedHash ) {
			$error['invalid_hash'] = sprintf( __( 'The hash security check did not pass. The hashes generated are different. '
				. 'Hash submitted: %s, hash received: %s.', WC_PayPoint::TEXT_DOMAIN ), $receivedHash, $hash );

			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: The hashes generated are different. Hash received:' . $receivedHash . ', generated hash:' . $hash );
		}

		// Check installation ID
		if ( $intInstID != $this->intInstID ) {
			$error['invalid_installationID'] = sprintf( __( 'The installation ID paid to does not match the one set in your store. Installation ID paid to: %s, Store installation ID: %s.', WC_PayPoint::TEXT_DOMAIN ), $intInstID, $this->intInstID );
			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: The installation ID paid to does not match the one set in your store. Installation ID paid to: ' . $intInstID . ', Store installation ID: ' . $this->intInstID );
		}

		// Check amounts
		if ( $amount != $order->get_total() ) {
			$error['invalid_amount'] = sprintf( __( 'The amount paid (%s) does not match the order total (%s).', WC_PayPoint::TEXT_DOMAIN ), $amount, $order->get_total() );
			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: The amount paidThe amount paid: ' . $amount . ', does not match the order total: ' . $order->get_total() );
		}

		// Check currency
		if ( $currency != $shopCurrency ) {
			$error['invalid_currency'] = sprintf( __( 'The currency paid in (%s) does not match your store currency (%s).', WC_PayPoint::TEXT_DOMAIN ), $currency, $shopCurrency );
			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: The currency paid in: ' . $currency . ', does not match your store currency: ' . $shopCurrency );
		}

		// Check invoice
		if ( $invoice != $order->order_key ) {
			$error['invalid_invoiceNumber'] = sprintf( __( 'The order invoice number does not match. Invoice received: %s, Order invoice: %s', WC_PayPoint::TEXT_DOMAIN ), $invoice, $order->order_key );
			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: The order invoice number does not match. Invoice received: ' . $invoice . ', Order invoice: ' . $order->order_key );
		}

		// Check if the transaction is the same test status as your store
		if ( ( 0 < $testmode ) && 'live' == $this->testmode ) {
			$error['invalid_Testmode'] = __( 'You cannot make test payments when your store is set to live mode.', WC_PayPoint::TEXT_DOMAIN );
			//Debug log
			if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Error: You cannot make test payments when your store is set to live mode.' );
		}

		if ( empty( $error ) ) {
			if ( 1 == $tranStatus ) {

				$order->add_order_note( sprintf( __( 'PayPoint payment completed. Transaction ID: %s', WC_PayPoint::TEXT_DOMAIN ), $transID ) );

				$order->payment_complete();

				// Remove cart
				WC_Compat_PayPoint::empty_cart();

				//Debug log
				if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Hosted + IMA Payment Completed' );

			} else {

				$order->add_order_note( sprintf( __( 'PayPoint payment failed. Bank fail message: %s', WC_PayPoint::TEXT_DOMAIN ), $failMessage ) );

				//Change status Failed
				$order->update_status( 'failed' );

				//Debug log
				if ( 'yes' == $this->debug ) WC_PayPoint::add_debug_log( 'Hosted + IMA Payment failed. Bank fail message: '. $failMessage );

			}
		} elseif ( ! empty( $error ) && 'yes' == $this->receive_error_logs ) {
			//Send error logs or checks if enabled
			$info = sprintf( __( 'Order# %s ', WC_PayPoint::TEXT_DOMAIN ), WC_PayPoint::get_order_number( $order ) );
			$this->email_paypoint_error_logs( $error, $posted, $info );
		}

		// Notify Paypoint response was received
		header( 'HTTP/1.0 200 OK' );
		flush();
		exit;
	}

	/**
	 * Check if this gateway is enabled and available in the user's country
	 */
	function is_available() {
		if ( $this->enabled == 'yes' ) {

			if ( 'hosted' == $this->integration ) {
				if ( ! $this->merchant_h ) { return false; }
				if ( ! $this->remotepassword_h ) { return false; }
				if ( ! $this->digest_key_h ) { return false; }
			} else {
				if ( ! $this->intInstID ) { return false; }
			}

			return true;
		}

		return false;
	}

	/**
	 * HOSTED+IMA PAYMENT
	 * Hash the main request parameters
	 */
	function hash_paypoint_params( $params ){
		$hash_string = $params['intInstID'] .':'. $params['strCartID'] .':'. $params['fltAmount'] .':'. $params['strCurrency'] .':'. $params['PT_invoice'] .':'. $params['intTestMode'];
		$hash = md5( $hash_string );
		return $hash;
	}

	/**
	 * Email the error logs
	 */
	function email_paypoint_error_logs( $error = array(), $posted = array(), $info = '' ) {
		$subject = sprintf( __( '[%s] PayPoint error logs.'), get_bloginfo('name') );
		$message = $info . PHP_EOL;
		$message .= '=======================================================' . PHP_EOL;
		if( ! empty( $error ) ) {
			$message .= 'Errors logged during the PayPoint payment response:' . PHP_EOL;
			$message .= print_r( $error, true );
			$message  .= '=======================================================' . PHP_EOL;
		}
		if( ! empty( $posted ) ) {
			$message .= 'All POST variables are:' . PHP_EOL;
			$message .= print_r( $posted, true );
			$message  .= '=======================================================' . PHP_EOL;
		}
		if( ! empty($this->emailto_error_logs) ) {
			$email = $this->emailto_error_logs;
		} else {
			$email = get_bloginfo('admin_email');
		}
		wp_mail( $email, $subject, $message );
	}
} // End WC_Gateway_PayPoint class
