<?php
/*
 * Plugin Name: WooCommerce PayPoint Payment Gateway
 * Plugin URI: http://woothemes.com/woocommerce
 * Description: Allows you to use <a href="http://www.paypoint.net">PayPoint</a> payment gateway with the WooCommerce plugin.
 * Version: 1.1
 * Author: VanboDevelops
 * Author URI: http://www.vanbodevelops.com
 *
 *	Copyright: (c) 2012 - 2014 VanboDevelops
 *	License: GNU General Public License v3.0
 *	License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */

if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Required functions
 */
if ( ! function_exists( 'woothemes_queue_update' ) )
    require_once( 'woo-includes/woo-functions.php' );

/**
 * Plugin updates
 */
woothemes_queue_update( plugin_basename( __FILE__ ), '6895043abb36a50a969f564452eed060', '18698' );

if ( ! is_woocommerce_active() ) { return; }

class WC_PayPoint {

	/**
	 * Text domain string. Constant
	 */
	const TEXT_DOMAIN = 'wc_paypoint';

	/**
	 * WC Logger object
	 * @var object
	 */
	private static $log;

	/**
	 * Plugin URL
	 * @var type
	 */
	private static $plugin_url;

	/**
	 * Plugin Path
	 * @var string
	 */
	private static $plugin_path;

	public function __construct() {

		// Add required files
		add_action( 'woocommerce_init', array( $this, 'load_gateway_files' ), 15 );

		add_action( 'init', array( $this, 'load_text_domain' ) );

		add_filter('woocommerce_payment_gateways', array( $this, 'add_paypoint_gateway' ) );

		// Add a 'Settings' link to the plugin action links
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'settings_support_link' ), 10, 4 );

	}

	/**
	 * Localisation
	 **/
	public function load_text_domain() {
		load_plugin_textdomain( self::TEXT_DOMAIN, false, self::plugin_path() . '/languages/' );
	}

	/**
	 * Add 'Settings' link to the plugin actions links
	 *
	 * @since 1.1
	 * @return array associative array of plugin action links
	 */
	public function settings_support_link( $actions, $plugin_file, $plugin_data, $context ) {

		$gateway = $this->get_gateway_class();

		return array_merge(
			array ( 'settings' => '<a href="' . WC_Compat_PayPoint::gateway_settings_page( $gateway ) . '">' . __( 'Settings', self::TEXT_DOMAIN ) . '</a>' ),
			$actions
		);
	}

	/**
	 * Get the correct gateway class name to load
	 *
	 * @since 1.1
	 * @return string Class name
	 */
	private function get_gateway_class() {
		return 'WC_Gateway_PayPoint';
	}

	/**
	 * Load gateway files
	 *
	 * @since 1.1
	 */
	public function load_gateway_files() {

		include_once( 'includes/class-wc-gateway-paypoint.php' );
		include_once( 'includes/class-wc-compat-paypoint.php' );

	}

	/**
	 * Savely get POST variables
	 *
	 * @since 1.1
	 * @param string $name POST variable name
	 * @return string The variable value
	 */
	public static function get_post( $name ) {
		if ( isset( $_POST[ $name ] ) ) {
		    return $_POST[ $name ];
		}
		return null;
	}

	/**
	 * Savely get GET variables
	 *
	 * @since 1.1
	 * @param string $name GET variable name
	 * @return string The variable value
	 */
	public static function get_get( $name ) {
		if ( isset( $_GET[ $name ] ) ) {
		    return $_GET[ $name ];
		}
		return null;
	}

	/**
	 * Add the gateway to WooCommerce
	 *
	 * @since 1.1
	 * @param array $methods
	 * @return array
	 */
	function add_paypoint_gateway( $methods ) {
		$methods[] = $this->get_gateway_class();

		return $methods;
	}

	/**
	 * Add debug log message
	 *
	 * @since 1.1
	 * @param string $message
	 */
	public static function add_debug_log( $message ) {
		if ( ! is_object( self::$log ) ) {
			self::$log = WC_Compat_PayPoint::get_wc_logger();
		}

		self::$log->add( 'PayPoint', $message );
	}

	/**
	 * Get the plugin url
	 *
	 * @since 1.1
	 * @return string
	 */
	public static function plugin_url() {
		if ( self::$plugin_url ) {
			return self::$plugin_url;
		}

		return self::$plugin_url = untrailingslashit( plugins_url( '/', __FILE__ ) );
	}

	/**
	 * Get the plugin path
	 *
	 * @since 1.1
	 * @return string
	 */
	public static function plugin_path() {
		if ( self::$plugin_path ) {
			return self::$plugin_path;
		}

		return self::$plugin_path = untrailingslashit( plugin_dir_path( __FILE__ ) );
	}

	/**
	 * Return the order number with stripped # or n° ( french translations )
	 *
	 * @since 1.1
	 * @param WC_Order $order
	 * @return string
	 */
	public static function get_order_number( WC_Order $order ) {
		return str_replace( array( '#', 'n°' ), '', $order->get_order_number() );
	}

} new WC_PayPoint();